@@ -58,7 +58,7 @@ class Manager(manager.Manager):
     driver_namespace = 'keystone.assignment'
     _provides_api = 'assignment_api'
 
-    _SYSTEM_SCOPE_TOKEN = 'system'
+    _SYSTEM_SCOPE_TOKEN = 'system'  # nosec
     _USER_SYSTEM = 'UserSystem'
     _GROUP_SYSTEM = 'GroupSystem'
     _PROJECT = 'project'
@@ -86,10 +86,10 @@ class Manager(manager.Manager):
         return [x['id'] for
                 x in PROVIDERS.identity_api.list_groups_for_user(user_id)]
 
-    def list_user_ids_for_project(self, tenant_id):
-        PROVIDERS.resource_api.get_project(tenant_id)
+    def list_user_ids_for_project(self, project_id):
+        PROVIDERS.resource_api.get_project(project_id)
         assignment_list = self.list_role_assignments(
-            project_id=tenant_id, effective=True)
+            project_id=project_id, effective=True)
         # Use set() to process the list to remove any duplicates
         return list(set([x['user_id'] for x in assignment_list]))
 
@@ -111,7 +111,7 @@ class Manager(manager.Manager):
                 )
 
     @MEMOIZE_COMPUTED_ASSIGNMENTS
-    def get_roles_for_user_and_project(self, user_id, tenant_id):
+    def get_roles_for_user_and_project(self, user_id, project_id):
         """Get the roles associated with a user within given project.
 
         This includes roles directly assigned to the user on the
@@ -123,9 +123,9 @@ class Manager(manager.Manager):
             exist.
 
         """
-        PROVIDERS.resource_api.get_project(tenant_id)
+        PROVIDERS.resource_api.get_project(project_id)
         assignment_list = self.list_role_assignments(
-            user_id=user_id, project_id=tenant_id, effective=True)
+            user_id=user_id, project_id=project_id, effective=True)
         # Use set() to process the list to remove any duplicates
         return list(set([x['role_id'] for x in assignment_list]))
 
@@ -185,22 +185,6 @@ class Manager(manager.Manager):
         role_ids = list(set([x['role_id'] for x in assignment_list]))
         return PROVIDERS.role_api.list_roles_from_ids(role_ids)
 
-    def ensure_default_role(self):
-        try:
-            PROVIDERS.role_api.get_role(CONF.member_role_id)
-        except exception.RoleNotFound:
-            LOG.info("Creating the default role %s "
-                     "because it does not exist.",
-                     CONF.member_role_id)
-            role = {'id': CONF.member_role_id,
-                    'name': CONF.member_role_name}
-            try:
-                PROVIDERS.role_api.create_role(CONF.member_role_id, role)
-            except exception.Conflict:
-                LOG.info("Creating the default role %s failed because it "
-                         "was already created",
-                         CONF.member_role_id)
-
     @notifications.role_assignment('created')
     def _add_role_to_user_and_project_adapter(self, role_id, user_id=None,
                                               group_id=None, domain_id=None,
@@ -216,9 +200,9 @@ class Manager(manager.Manager):
         PROVIDERS.role_api.get_role(role_id)
         self.driver.add_role_to_user_and_project(user_id, project_id, role_id)
 
-    def add_role_to_user_and_project(self, user_id, tenant_id, role_id):
+    def add_role_to_user_and_project(self, user_id, project_id, role_id):
         self._add_role_to_user_and_project_adapter(
-            role_id, user_id=user_id, project_id=tenant_id)
+            role_id, user_id=user_id, project_id=project_id)
         COMPUTED_ASSIGNMENTS_REGION.invalidate()
 
     # TODO(henry-nash): We might want to consider list limiting this at some
@@ -287,9 +271,9 @@ class Manager(manager.Manager):
             role_id, group_id, user_id, project_id, domain_id
         )
 
-    def remove_role_from_user_and_project(self, user_id, tenant_id, role_id):
+    def remove_role_from_user_and_project(self, user_id, project_id, role_id):
         self._remove_role_from_user_and_project_adapter(
-            role_id, user_id=user_id, project_id=tenant_id)
+            role_id, user_id=user_id, project_id=project_id)
         COMPUTED_ASSIGNMENTS_REGION.invalidate()
 
     def _invalidate_token_cache(self, role_id, group_id, user_id, project_id,
@@ -321,7 +305,8 @@ class Manager(manager.Manager):
     @notifications.role_assignment('created')
     def create_grant(self, role_id, user_id=None, group_id=None,
                      domain_id=None, project_id=None,
-                     inherited_to_projects=False, context=None):
+                     inherited_to_projects=False,
+                     initiator=None):
         role = PROVIDERS.role_api.get_role(role_id)
         if domain_id:
             PROVIDERS.resource_api.get_domain(domain_id)
@@ -371,7 +356,8 @@ class Manager(manager.Manager):
     @notifications.role_assignment('deleted')
     def delete_grant(self, role_id, user_id=None, group_id=None,
                      domain_id=None, project_id=None,
-                     inherited_to_projects=False, context=None):
+                     inherited_to_projects=False,
+                     initiator=None):
 
         # check if role exist before any processing
         PROVIDERS.role_api.get_role(role_id)
@@ -682,9 +668,8 @@ class Manager(manager.Manager):
                         _make_implied_ref_copy(
                             next_ref, implied_role['implied_role_id']))
                     if implied_ref in checked_role_refs:
-                        msg = ('Circular reference found '
-                               'role inference rules - %(prior_role_id)s.')
-                        LOG.error(msg, {'prior_role_id': next_ref['role_id']})
+                        # Avoid traversing a cycle
+                        continue
                     else:
                         ref_results.append(implied_ref)
                         role_refs_to_check.append(implied_ref)
@@ -941,9 +926,11 @@ class Manager(manager.Manager):
                     a['system'] = {'all': True}
                     system_assignments.append(a)
 
-            for i, assignment in enumerate(system_assignments):
-                if role_id and role_id != assignment['role_id']:
-                    system_assignments.pop(i)
+            if role_id:
+                system_assignments = [
+                    sa for sa in system_assignments
+                    if role_id == sa['role_id']
+                ]
 
         assignments = []
         for assignment in itertools.chain(
