@@ -16,6 +16,7 @@
 
 import abc
 
+import flask
 import six
 
 from keystone.auth.plugins import base
@@ -31,34 +32,29 @@ PROVIDERS = provider_api.ProviderAPIs
 
 @six.add_metaclass(abc.ABCMeta)
 class Base(base.AuthMethodHandler):
-    def authenticate(self, request, auth_payload,):
+    def authenticate(self, auth_payload):
         """Use REMOTE_USER to look up the user in the identity backend.
 
         The user_id from the actual user from the REMOTE_USER env variable is
         placed in the response_data.
         """
         response_data = {}
-        if not request.remote_user:
+        if not flask.request.remote_user:
             msg = _('No authenticated user')
             raise exception.Unauthorized(msg)
 
         try:
-            user_ref = self._authenticate(request)
+            user_ref = self._authenticate()
         except Exception:
-            msg = _('Unable to lookup user %s') % request.remote_user
+            msg = _('Unable to lookup user %s') % flask.request.remote_user
             raise exception.Unauthorized(msg)
 
         response_data['user_id'] = user_ref['id']
-        auth_type = (request.auth_type or '').lower()
-
-        if 'kerberos' in CONF.token.bind and auth_type == 'negotiate':
-            response_data.setdefault('bind', {})['kerberos'] = user_ref['name']
-
         return base.AuthHandlerResponse(status=True, response_body=None,
                                         response_data=response_data)
 
     @abc.abstractmethod
-    def _authenticate(self, request):
+    def _authenticate(self):
         """Look up the user in the identity backend.
 
         Return user_ref
@@ -67,36 +63,35 @@ class Base(base.AuthMethodHandler):
 
 
 class DefaultDomain(Base):
-    def _authenticate(self, request):
+    def _authenticate(self):
         """Use remote_user to look up the user in the identity backend."""
         return PROVIDERS.identity_api.get_user_by_name(
-            request.remote_user,
+            flask.request.remote_user,
             CONF.identity.default_domain_id)
 
 
 class Domain(Base):
-    def _authenticate(self, request):
+    def _authenticate(self):
         """Use remote_user to look up the user in the identity backend.
 
         The domain will be extracted from the REMOTE_DOMAIN environment
         variable if present. If not, the default domain will be used.
         """
-        if request.remote_domain:
-            ref = PROVIDERS.resource_api.get_domain_by_name(
-                request.remote_domain
-            )
+        remote_domain = flask.request.environ.get('REMOTE_DOMAIN')
+        if remote_domain:
+            ref = PROVIDERS.resource_api.get_domain_by_name(remote_domain)
             domain_id = ref['id']
         else:
             domain_id = CONF.identity.default_domain_id
 
-        return PROVIDERS.identity_api.get_user_by_name(request.remote_user,
-                                                       domain_id)
+        return PROVIDERS.identity_api.get_user_by_name(
+            flask.request.remote_user, domain_id)
 
 
 class KerberosDomain(Domain):
     """Allows `kerberos` as a method."""
 
-    def _authenticate(self, request):
-        if request.auth_type != 'Negotiate':
+    def _authenticate(self):
+        if flask.request.environ.get('AUTH_TYPE') != 'Negotiate':
             raise exception.Unauthorized(_("auth_type is not Negotiate"))
-        return super(KerberosDomain, self)._authenticate(request)
+        return super(KerberosDomain, self)._authenticate()
