@@ -17,20 +17,6 @@ from oslo_log import versionutils
 
 from keystone.conf import utils
 
-bind = cfg.ListOpt(
-    'bind',
-    default=[],
-    deprecated_since=versionutils.deprecated.PIKE,
-    deprecated_for_removal=True,
-    help=utils.fmt("""
-This is a list of external authentication mechanisms which should add token
-binding metadata to tokens, such as `kerberos` or `x509`. Note that this option
-is deprecated as keystone no longer supports binding metadata to tokens
-directly. This option is silently ignored and will be removed in the future.
-This option no longer has any impact on the behavior of tokens and can be
-removed.
-"""))
-
 expiration = cfg.IntOpt(
     'expiration',
     default=3600,
@@ -52,10 +38,15 @@ provider = cfg.StrOpt(
     help=utils.fmt("""
 Entry point for the token provider in the `keystone.token.provider` namespace.
 The token provider controls the token construction, validation, and revocation
-operations. Keystone includes `fernet` token provider.
-`fernet` tokens do not need to be persisted at all, but require that you run
-`keystone-manage fernet_setup` (also see the `keystone-manage fernet_rotate`
-command).
+operations. Supported upstream providers are `fernet` and `jws`. Neither
+`fernet` or `jws` tokens require persistence and both require additional setup.
+If using `fernet`, you're required to run `keystone-manage fernet_setup`, which
+creates symmetric keys used to encrypt tokens. If using `jws`, you're required
+to generate an ECDSA keypair using a SHA-256 hash algorithm for signing and
+validating token, which can be done with `keystone-manage create_jws_keypair`.
+Note that `fernet` tokens are encrypted and `jws` tokens are only signed.
+Please be sure to consider this if your deployment has security requirements
+regarding payload contents used to generate token IDs.
 """))
 
 caching = cfg.BoolOpt(
@@ -118,10 +109,21 @@ other role assignments.
 cache_on_issue = cfg.BoolOpt(
     'cache_on_issue',
     default=True,
+    deprecated_since=versionutils.deprecated.STEIN,
+    deprecated_reason=utils.fmt("""
+Keystone already exposes a configuration option for caching tokens. Having a
+separate configuration option to cache tokens when they are issued is
+redundant, unnecessarily complicated, and is misleading if token caching is
+disabled because tokens will still be pre-cached by default when they are
+issued. The ability to pre-cache tokens when they are issued is going to rely
+exclusively on the ``keystone.conf [token] caching`` option in the future.
+"""),
+    deprecated_for_removal=True,
     help=utils.fmt("""
 Enable storing issued token data to token validation cache so that first token
 validation doesn't actually cause full validation cycle. This option has no
-effect unless global caching and token caching are enabled.
+effect unless global caching is enabled and will still cache tokens even if
+`[token] caching = False`.
 """))
 
 allow_expired_window = cfg.IntOpt(
