@@ -12,9 +12,8 @@
 
 """Unified in-memory token model."""
 
-import itertools
-
 from oslo_log import log
+from oslo_serialization import jsonutils
 from oslo_serialization import msgpackutils
 from oslo_utils import reflection
 import six
@@ -254,6 +253,7 @@ class TokenModel(object):
         roles = []
         groups = PROVIDERS.identity_api.list_groups_for_user(self.user_id)
         all_group_roles = []
+        assignments = []
         for group in groups:
             group_roles = (
                 PROVIDERS.assignment_api.list_system_grants_for_group(
@@ -262,10 +262,25 @@ class TokenModel(object):
             )
             for role in group_roles:
                 all_group_roles.append(role)
+                assignment = {'group_id': group['id'], 'role_id': role['id']}
+                assignments.append(assignment)
         user_roles = PROVIDERS.assignment_api.list_system_grants_for_user(
             self.user_id
         )
-        for role in itertools.chain(all_group_roles, user_roles):
+        for role in user_roles:
+            assignment = {'user_id': self.user_id, 'role_id': role['id']}
+            assignments.append(assignment)
+
+        # NOTE(lbragstad): The whole reason we need to build out a list of
+        # "assignments" as opposed to just using the nice list of roles we
+        # already have is because the add_implied_roles() method operates on a
+        # list of assignment dictionaries (containing role_id,
+        # user_id/group_id, project_id, et cetera). That method could probably
+        # be fixed to be more clear by operating on actual roles instead of
+        # just assignments.
+        assignments = PROVIDERS.assignment_api.add_implied_roles(assignments)
+        for assignment in assignments:
+            role = PROVIDERS.role_api.get_role(assignment['role_id'])
             roles.append({'id': role['id'], 'name': role['name']})
 
         return roles
@@ -311,6 +326,21 @@ class TokenModel(object):
 
         return roles
 
+    def _get_oauth_roles(self):
+        roles = []
+        access_token_roles = self.access_token['role_ids']
+        access_token_roles = [
+            {'role_id': r} for r in jsonutils.loads(access_token_roles)]
+        effective_access_token_roles = (
+            PROVIDERS.assignment_api.add_implied_roles(access_token_roles)
+        )
+        user_roles = [r['id'] for r in self._get_project_roles()]
+        for role in effective_access_token_roles:
+            if role['role_id'] in user_roles:
+                role = PROVIDERS.role_api.get_role(role['role_id'])
+                roles.append({'id': role['id'], 'name': role['name']})
+        return roles
+
     def _get_federated_roles(self):
         roles = []
         group_ids = [group['id'] for group in self.federated_groups]
@@ -395,14 +425,16 @@ class TokenModel(object):
     def _get_application_credential_roles(self):
         roles = []
         app_cred_roles = self.application_credential['roles']
+        assignment_list = PROVIDERS.assignment_api.list_role_assignments(
+            user_id=self.user_id,
+            project_id=self.project_id,
+            domain_id=self.domain_id,
+            effective=True)
+        user_roles = list(set([x['role_id'] for x in assignment_list]))
+
         for role in app_cred_roles:
-            try:
-                r = PROVIDERS.assignment_api.get_grant(
-                    role['id'], user_id=self.user_id,
-                    domain_id=self.domain_id, project_id=self.project_id)
-                roles.append({'id': r['id'], 'name': r['name']})
-            except exception.RoleAssignmentNotFound:
-                pass
+            if role['id'] in user_roles:
+                roles.append({'id': role['id'], 'name': role['name']})
 
         return roles
 
@@ -412,6 +444,8 @@ class TokenModel(object):
             roles = self._get_system_roles()
         elif self.trust_scoped:
             roles = self._get_trust_roles()
+        elif self.oauth_scoped:
+            roles = self._get_oauth_roles()
         elif self.is_federated and not self.unscoped:
             roles = self._get_federated_roles()
         elif self.domain_scoped:
@@ -426,15 +460,19 @@ class TokenModel(object):
 
     def _validate_token_resources(self):
         if self.project and not self.project.get('enabled'):
-            msg = _('Unable to validate token because project %(id)s is '
-                    'disabled') % {'id': self.project_id}
+            msg = ('Unable to validate token because project %(id)s is '
+                   'disabled') % {'id': self.project_id}
+            tr_msg = _('Unable to validate token because project %(id)s is '
+                       'disabled') % {'id': self.project_id}
             LOG.warning(msg)
-            raise exception.ProjectNotFound(msg)
+            raise exception.ProjectNotFound(tr_msg)
         if self.project and not self.project_domain.get('enabled'):
-            msg = _('Unable to validate token because domain %(id)s is '
-                    'disabled') % {'id': self.project_domain['id']}
+            msg = ('Unable to validate token because domain %(id)s is '
+                   'disabled') % {'id': self.project_domain['id']}
+            tr_msg = _('Unable to validate token because domain %(id)s is '
+                       'disabled') % {'id': self.project_domain['id']}
             LOG.warning(msg)
-            raise exception.DomainNotFound(msg)
+            raise exception.DomainNotFound(tr_msg)
 
     def _validate_token_user(self):
         if self.trust_scoped:
@@ -461,34 +499,43 @@ class TokenModel(object):
                 raise exception.Forbidden(_('Trustor is disabled.'))
 
         if not self.user_domain.get('enabled'):
-            msg = _('Unable to validate token because domain %(id)s is '
-                    'disabled') % {'id': self.user_domain['id']}
+            msg = ('Unable to validate token because domain %(id)s is '
+                   'disabled') % {'id': self.user_domain['id']}
+            tr_msg = _('Unable to validate token because domain %(id)s is '
+                       'disabled') % {'id': self.user_domain['id']}
             LOG.warning(msg)
-            raise exception.DomainNotFound(msg)
+            raise exception.DomainNotFound(tr_msg)
 
     def _validate_system_scope(self):
         if self.system_scoped and not self.roles:
-            msg = _(
-                'User %(user_id)s has no access to the system'
-            ) % {'user_id': self.user_id}
+            msg = ('User %(user_id)s has no access to the system'
+                   ) % {'user_id': self.user_id}
+            tr_msg = _('User %(user_id)s has no access to the system'
+                       ) % {'user_id': self.user_id}
             LOG.debug(msg)
-            raise exception.Unauthorized(msg)
+            raise exception.Unauthorized(tr_msg)
 
     def _validate_domain_scope(self):
         if self.domain_scoped and not self.roles:
-            msg = _(
+            msg = (
+                'User %(user_id)s has no access to domain %(domain_id)s'
+            ) % {'user_id': self.user_id, 'domain_id': self.domain_id}
+            tr_msg = _(
                 'User %(user_id)s has no access to domain %(domain_id)s'
             ) % {'user_id': self.user_id, 'domain_id': self.domain_id}
             LOG.debug(msg)
-            raise exception.Unauthorized(msg)
+            raise exception.Unauthorized(tr_msg)
 
     def _validate_project_scope(self):
         if self.project_scoped and not self.roles:
-            msg = _(
+            msg = (
+                'User %(user_id)s has no access to project %(project_id)s'
+            ) % {'user_id': self.user_id, 'project_id': self.project_id}
+            tr_msg = _(
                 'User %(user_id)s has no access to project %(project_id)s'
             ) % {'user_id': self.user_id, 'project_id': self.project_id}
             LOG.debug(msg)
-            raise exception.Unauthorized(msg)
+            raise exception.Unauthorized(tr_msg)
 
     def _validate_trust_scope(self):
         trust_roles = []
