@@ -19,6 +19,7 @@ import functools
 import inspect
 import socket
 
+import flask
 from oslo_log import log
 import oslo_messaging
 from oslo_utils import reflection
@@ -27,9 +28,11 @@ from pycadf import cadftaxonomy as taxonomy
 from pycadf import cadftype
 from pycadf import credential
 from pycadf import eventfactory
+from pycadf import host
 from pycadf import reason
 from pycadf import resource
 
+from keystone.common import context
 from keystone.common import provider_api
 from keystone.common import utils
 import keystone.conf
@@ -70,18 +73,45 @@ SAML_AUDIT_TYPE = 'http://docs.oasis-open.org/security/saml/v2.0'
 _SUBSCRIBERS = {}
 _notifier = None
 SERVICE = 'identity'
+PROVIDERS = provider_api.ProviderAPIs
 
+ROOT_DOMAIN = '<<keystone.domain.root>>'
 
 CONF = keystone.conf.CONF
 
 # NOTE(morganfainberg): Special case notifications that are only used
 # internally for handling token persistence token deletions
-INVALIDATE_TOKEN_CACHE = 'invalidate_token_cache'
+INVALIDATE_TOKEN_CACHE = 'invalidate_token_cache'  # nosec
 PERSIST_REVOCATION_EVENT_FOR_USER = 'persist_revocation_event_for_user'
 REMOVE_APP_CREDS_FOR_USER = 'remove_application_credentials_for_user'
 DOMAIN_DELETED = 'domain_deleted'
 
 
+def build_audit_initiator():
+    """A pyCADF initiator describing the current authenticated context."""
+    pycadf_host = host.Host(address=flask.request.remote_addr,
+                            agent=str(flask.request.user_agent))
+    initiator = resource.Resource(typeURI=taxonomy.ACCOUNT_USER,
+                                  host=pycadf_host)
+    oslo_context = flask.request.environ.get(context.REQUEST_CONTEXT_ENV)
+    if oslo_context.user_id:
+        initiator.id = utils.resource_uuid(oslo_context.user_id)
+        initiator.user_id = oslo_context.user_id
+
+    if oslo_context.project_id:
+        initiator.project_id = oslo_context.project_id
+
+    if oslo_context.domain_id:
+        initiator.domain_id = oslo_context.domain_id
+
+    initiator.request_id = oslo_context.request_id
+
+    if oslo_context.global_request_id:
+        initiator.global_request_id = oslo_context.global_request_id
+
+    return initiator
+
+
 class Audit(object):
     """Namespace for audit notification functions.
 
@@ -116,7 +146,8 @@ class Audit(object):
             operation,
             resource_type,
             resource_id,
-            actor_dict,
+            initiator=initiator,
+            actor_dict=actor_dict,
             public=public)
 
         if CONF.notification_format == 'cadf' and public:
@@ -247,9 +278,10 @@ def register_event_callback(event, resource_type, callbacks):
 
     for callback in callbacks:
         if not callable(callback):
-            msg = _('Method not callable: %s') % callback
+            msg = 'Method not callable: %s' % callback
+            tr_msg = _('Method not callable: %s') % callback
             LOG.error(msg)
-            raise TypeError(msg)
+            raise TypeError(tr_msg)
         _SUBSCRIBERS.setdefault(event, {}).setdefault(resource_type, set())
         _SUBSCRIBERS[event][resource_type].add(callback)
 
@@ -396,6 +428,13 @@ def _create_cadf_payload(operation, resource_type, resource_id,
         target_uri = taxonomy.UNKNOWN
     else:
         target_uri = CADF_TYPE_MAP.get(resource_type)
+
+    # TODO(gagehugo): The root domain ID is typically hidden, there isn't a
+    # reason to emit a notification for it. Once we expose the root domain
+    # (and handle the CADF UUID), remove this.
+    if resource_id == ROOT_DOMAIN:
+        return
+
     target = resource.Resource(typeURI=target_uri,
                                id=resource_id)
 
@@ -407,8 +446,8 @@ def _create_cadf_payload(operation, resource_type, resource_id,
                              target, event_type, reason=reason, **audit_kwargs)
 
 
-def _send_notification(operation, resource_type, resource_id, actor_dict=None,
-                       public=True):
+def _send_notification(operation, resource_type, resource_id, initiator=None,
+                       actor_dict=None, public=True):
     """Send notification to inform observers about the affected resource.
 
     This method doesn't raise an exception when sending the notification fails.
@@ -416,6 +455,7 @@ def _send_notification(operation, resource_type, resource_id, actor_dict=None,
     :param operation: operation being performed (created, updated, or deleted)
     :param resource_type: type of resource being operated on
     :param resource_id: ID of resource being operated on
+    :param initiator: representation of the user that created the request
     :param actor_dict: a dictionary containing the actor's ID and type
     :param public:  if True (default), the event will be sent
                     to the notifier API.
@@ -429,6 +469,12 @@ def _send_notification(operation, resource_type, resource_id, actor_dict=None,
         payload['actor_type'] = actor_dict['type']
         payload['actor_operation'] = actor_dict['actor_operation']
 
+    if initiator:
+        payload['request_id'] = initiator.request_id
+        global_request_id = getattr(initiator, 'global_request_id', None)
+        if global_request_id:
+            payload['global_request_id'] = global_request_id
+
     notify_event_callbacks(SERVICE, resource_type, operation, payload)
 
     # Only send this notification if the 'basic' format is used, otherwise
@@ -483,6 +529,7 @@ def _get_request_audit_info(context, user_id=None):
     if user_id:
         initiator.user_id = user_id
         initiator.id = utils.resource_uuid(user_id)
+        initiator = _add_username_to_initiator(initiator)
 
     if project_id:
         initiator.project_id = project_id
@@ -514,14 +561,15 @@ class CadfNotificationWrapper(object):
 
     def __call__(self, f):
         @functools.wraps(f)
-        def wrapper(wrapped_self, request, user_id, *args, **kwargs):
+        def wrapper(wrapped_self, user_id, *args, **kwargs):
             """Will always send a notification."""
             target = resource.Resource(typeURI=taxonomy.ACCOUNT_USER)
-            initiator = request.audit_initiator
+            initiator = build_audit_initiator()
             initiator.user_id = user_id
+            initiator = _add_username_to_initiator(initiator)
             initiator.id = utils.resource_uuid(user_id)
             try:
-                result = f(wrapped_self, request, user_id, *args, **kwargs)
+                result = f(wrapped_self, user_id, *args, **kwargs)
             except (exception.AccountLocked,
                     exception.PasswordExpired) as ex:
                 # Send a CADF event with a reason for PCI-DSS related
@@ -612,9 +660,7 @@ class CadfRoleAssignmentNotificationWrapper(object):
             call_args = inspect.getcallargs(
                 f, wrapped_self, role_id, *args, **kwargs)
             inherited = call_args['inherited_to_projects']
-            context = call_args['context']
-
-            initiator = _get_request_audit_info(context)
+            initiator = call_args.get('initiator', None)
             target = resource.Resource(typeURI=taxonomy.ACCOUNT_USER)
 
             audit_kwargs = {}
@@ -649,15 +695,13 @@ class CadfRoleAssignmentNotificationWrapper(object):
         return wrapper
 
 
-def send_saml_audit_notification(action, request, user_id, group_ids,
+def send_saml_audit_notification(action, user_id, group_ids,
                                  identity_provider, protocol, token_id,
                                  outcome):
     """Send notification to inform observers about SAML events.
 
     :param action: Action being audited
     :type action: str
-    :param request: Current request to collect request info from
-    :type request: keystone.common.request.Request
     :param user_id: User ID from Keystone token
     :type user_id: str
     :param group_ids: List of Group IDs from Keystone token
@@ -671,7 +715,7 @@ def send_saml_audit_notification(action, request, user_id, group_ids,
     :param outcome: One of :class:`pycadf.cadftaxonomy`
     :type outcome: str
     """
-    initiator = request.audit_initiator
+    initiator = build_audit_initiator()
     target = resource.Resource(typeURI=taxonomy.ACCOUNT_USER)
     audit_type = SAML_AUDIT_TYPE
     user_id = user_id or taxonomy.UNKNOWN
@@ -721,6 +765,8 @@ def _send_audit_notification(action, initiator, outcome, target,
             service_id = i['id']
             break
 
+    initiator = _add_username_to_initiator(initiator)
+
     event = eventfactory.EventFactory().new_event(
         eventType=cadftype.EVENTTYPE_ACTIVITY,
         outcome=outcome,
@@ -778,6 +824,19 @@ def _check_notification_opt_out(event_type, outcome):
     return False
 
 
+def _add_username_to_initiator(initiator):
+    """Add the username to the initiator if missing."""
+    if hasattr(initiator, 'username'):
+        return initiator
+    try:
+        user_ref = PROVIDERS.identity_api.get_user(initiator.user_id)
+        initiator.username = user_ref['name']
+    except (exception.UserNotFound, AttributeError):
+        # Either user not found or no user_id, move along
+        pass
+
+    return initiator
+
 emit_event = CadfNotificationWrapper
 
 
