@@ -11,8 +11,10 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from keystone.tests.unit.ksfixtures.access_rules_config import AccessRulesConfig  # noqa
 from keystone.tests.unit.ksfixtures.auth_plugins import ConfigAuthPlugins  # noqa
 from keystone.tests.unit.ksfixtures.backendloader import BackendLoader  # noqa
 from keystone.tests.unit.ksfixtures.cache import Cache  # noqa
+from keystone.tests.unit.ksfixtures.jws_key_repository import JWSKeyRepository  # noqa
 from keystone.tests.unit.ksfixtures.key_repository import KeyRepository  # noqa
 from keystone.tests.unit.ksfixtures.policy import Policy  # noqa
