@@ -16,6 +16,8 @@
 # test_hacking_checks test cases.
 # flake8: noqa
 
+import sys
+
 import fixtures
 
 
@@ -73,11 +75,18 @@ class HackingCode(fixtures.Fixture):
             (25, 14, 'K001'),
             (25, 36, 'K001'),
             (28, 10, 'K001'),
-            (28, 27, 'K001'),
+            (28, 26, 'K001'),
             (29, 21, 'K001'),
-            (32, 11, 'K001'),
+            (32, 10, 'K001'),
         ]}
 
+    # NOTE(browne): This is gross, but in Python 3.4 and earlier, the ast
+    # module returns the incorrect col_offset for two of the defined functions
+    # in the code sample above.
+    if sys.version_info < (3, 5):
+        mutable_default_args['expected_errors'][12] = (28, 27, 'K001')
+        mutable_default_args['expected_errors'][14] = (32, 11, 'K001')
+
     comments_begin_with_space = {
         'code': """
             # This is a good comment
@@ -114,9 +123,9 @@ class HackingCode(fixtures.Fixture):
 
     dict_constructor = {
         'code': """
-            lower_res = {k.lower(): v for k, v in six.iteritems(res[1])}
+            lower_res = {k.lower(): v for k, v in res[1].items()}
             fool = dict(a='a', b='b')
-            lower_res = dict((k.lower(), v) for k, v in six.iteritems(res[1]))
+            lower_res = dict((k.lower(), v) for k, v in res[1].items())
             attrs = dict([(k, _from_json(v))])
             dict([[i,i] for i in range(3)])
             dict(({1:2}))
@@ -128,18 +137,20 @@ class HackingCode(fixtures.Fixture):
         ]}
 
 
-class HackingLogging(fixtures.Fixture):
+class HackingTranslations(fixtures.Fixture):
+    """Fixtures for checking translation rules.
+
+    1. Exception messages should be translated
+    2. Logging messages should not be translated
+    3. If a message is used for both an exception and logging it
+       should be translated
+    """
 
     shared_imports = """
                 import logging
                 import logging as stlib_logging
                 from keystone.i18n import _
                 from keystone.i18n import _ as oslo_i18n
-                from keystone.i18n import _LC
-                from keystone.i18n import _LE
-                from keystone.i18n import _LE as error_hint
-                from keystone.i18n import _LI
-                from keystone.i18n import _LW
                 from oslo_log import log
                 from oslo_log import log as oslo_logging
     """
@@ -153,11 +164,10 @@ class HackingLogging(fixtures.Fixture):
                 class C:
                     def __init__(self):
                         LOG.warning(oslo_i18n('text', {}))
-                        LOG.warning(_LW('text', {}))
             """,
             'expected_errors': [
-                (3, 9, 'K006'),
-                (6, 20, 'K006'),
+                (3, 9, 'K005'),
+                (6, 20, 'K005'),
             ],
         },
         {
@@ -170,12 +180,9 @@ class HackingLogging(fixtures.Fixture):
                         self.L.warning(
                             _('text'), {}
                         )
-                        self.L.warning(
-                            _LW('text'), {}
-                        )
             """,
             'expected_errors': [
-                (7, 12, 'K006'),
+                (7, 12, 'K005'),
             ],
         },
         {
@@ -183,10 +190,9 @@ class HackingLogging(fixtures.Fixture):
                 # oslo logging and specifying a logger
                 L = log.getLogger(__name__)
                 L.error(oslo_i18n('text'))
-                L.error(error_hint('text'))
             """,
             'expected_errors': [
-                (3, 8, 'K006'),
+                (3, 8, 'K005'),
             ],
         },
         {
@@ -196,10 +202,9 @@ class HackingLogging(fixtures.Fixture):
                     def __init__(self):
                         self.LOG = oslo_logging.getLogger()
                         self.LOG.critical(_('text'))
-                        self.LOG.critical(_LC('text'))
             """,
             'expected_errors': [
-                (5, 26, 'K006'),
+                (5, 26, 'K005'),
             ],
         },
         {
@@ -208,48 +213,35 @@ class HackingLogging(fixtures.Fixture):
                 # translation on a separate line
                 msg = _('text')
                 LOG.exception(msg)
-                msg = _LE('text')
-                LOG.exception(msg)
             """,
             'expected_errors': [
-                (4, 14, 'K006'),
+                (4, 14, 'K005'),
             ],
         },
         {
             'code': """
-                LOG = logging.getLogger()
-
-                # ensure the correct helper is being used
-                LOG.warning(_LI('this should cause an error'))
-
-                # debug should not allow any helpers either
-                LOG.debug(_LI('this should cause an error'))
-            """,
-            'expected_errors': [
-                (4, 12, 'K006'),
-                (7, 10, 'K005'),
-            ],
-        },
-        {
-            'code': """
-                # this should not be an error
+                # this should be an error even if it'll be raised later.
                 L = log.getLogger(__name__)
                 msg = _('text')
                 L.warning(msg)
                 raise Exception(msg)
             """,
-            'expected_errors': [],
+            'expected_errors': [
+                (4, 10, 'K005'),
+            ],
         },
         {
             'code': """
                 L = log.getLogger(__name__)
                 def f():
                     msg = _('text')
-                    L2.warning(msg)
+                    L.warning(msg)
                     something = True  # add an extra statement here
                     raise Exception(msg)
             """,
-            'expected_errors': [],
+            'expected_errors': [
+                (4, 14, 'K005'),
+            ],
         },
         {
             'code': """
@@ -260,7 +252,7 @@ class HackingLogging(fixtures.Fixture):
                     raise Exception('some other message')
             """,
             'expected_errors': [
-                (4, 16, 'K006'),
+                (4, 16, 'K005'),
             ],
         },
         {
@@ -274,6 +266,7 @@ class HackingLogging(fixtures.Fixture):
                 raise Exception(msg)
             """,
             'expected_errors': [
+                (6, 12, 'K005'),
             ],
         },
         {
@@ -286,18 +279,7 @@ class HackingLogging(fixtures.Fixture):
                 LOG.warning(msg)
             """,
             'expected_errors': [
-                (6, 12, 'K006'),
-            ],
-        },
-        {
-            'code': """
-                LOG = log.getLogger(__name__)
-                msg = _LW('text')
-                LOG.warning(msg)
-                raise Exception(msg)
-            """,
-            'expected_errors': [
-                (3, 12, 'K007'),
+                (6, 12, 'K005'),
             ],
         },
         {
@@ -313,22 +295,13 @@ class HackingLogging(fixtures.Fixture):
         {
             'code': """
                 LOG = log.getLogger(__name__)
-                msg = _LW('hello %s') % 'world'
+                msg = _('hello %s') % 'world'
                 LOG.warning(msg)
-                raise Exception(msg)
             """,
             'expected_errors': [
-                (3, 12, 'K007'),
+                (3, 12, 'K005'),
             ],
         },
-        {
-            'code': """
-                LOG = log.getLogger(__name__)
-                msg = _LW('hello %s') % 'world'
-                LOG.warning(msg)
-            """,
-            'expected_errors': [],
-        },
         {
             'code': """
                 # this should not be an error
@@ -341,77 +314,19 @@ class HackingLogging(fixtures.Fixture):
             """,
             'expected_errors': [],
         },
-    ]
-
-    assert_not_using_deprecated_warn = {
-        'code': """
-                # Logger.warn has been deprecated in Python3 in favor of
-                # Logger.warning
+        {
+            'code': """
+                # this should not be an error
                 LOG = log.getLogger(__name__)
-                LOG.warn(_LW('text'))
-        """,
-        'expected_errors': [
-            (4, 9, 'K009'),
-        ],
-    }
-
-    assert_no_translations_for_debug_logging = {
-        'code': """
-                # stdlib logging
-                L0 = logging.getLogger()
-                L0.debug(_('text'))
-                class C:
-                    def __init__(self):
-                        L0.debug(oslo_i18n('text', {}))
-
-                # stdlib logging w/ alias and specifying a logger
-                class C:
-                    def __init__(self):
-                        self.L1 = logging.getLogger(__name__)
-                    def m(self):
-                        self.L1.debug(
-                            _('text'), {}
-                        )
-
-                # oslo logging and specifying a logger
-                L2 = logging.getLogger(__name__)
-                L2.debug(oslo_i18n('text'))
-
-                # oslo logging w/ alias
-                class C:
-                    def __init__(self):
-                        self.L3 = oslo_logging.getLogger()
-                        self.L3.debug(_('text'))
-
-                # translation on a separate line
-                msg = _('text')
-                L2.debug(msg)
-
-                # this should not fail
-                if True:
-                    msg = _('message %s') % X
-                    L2.error(msg)
-                    raise TypeError(msg)
-                if True:
-                    msg = 'message'
-                    L2.debug(msg)
-
-                # this should not fail
-                if True:
-                    if True:
-                        msg = _('message')
-                    else:
-                        msg = _('message')
-                    L2.debug(msg)
-                    raise Exception(msg)
-        """,
-        'expected_errors': [
-            (3, 9, 'K005'),
-            (6, 17, 'K005'),
-            (14, 12, 'K005'),
-            (19, 9, 'K005'),
-            (25, 22, 'K005'),
-            (29, 9, 'K005'),
-        ]
-    }
-
+                try:
+                    pass
+                except AssertionError as e:
+                    msg = _('some message')
+                    LOG.warning(msg)
+                    raise exception.Unauthorized(message=msg)
+            """,
+            'expected_errors': [
+                (7, 16, 'K005'),
+            ],
+        },
+    ]
