@@ -21,6 +21,7 @@ from keystone.common import driver_hints
 from keystone.common import provider_api
 import keystone.conf
 from keystone import exception
+from keystone.resource.backends import sql as resource_sql
 from keystone.tests import unit
 from keystone.tests.unit import default_fixtures
 from keystone.tests.unit import utils as test_utils
@@ -35,8 +36,8 @@ class ResourceTests(object):
     domain_count = len(default_fixtures.DOMAINS)
 
     def test_get_project(self):
-        tenant_ref = PROVIDERS.resource_api.get_project(self.tenant_bar['id'])
-        self.assertDictEqual(self.tenant_bar, tenant_ref)
+        project_ref = PROVIDERS.resource_api.get_project(self.project_bar['id'])
+        self.assertDictEqual(self.project_bar, project_ref)
 
     def test_get_project_returns_not_found(self):
         self.assertRaises(exception.ProjectNotFound,
@@ -44,10 +45,10 @@ class ResourceTests(object):
                           uuid.uuid4().hex)
 
     def test_get_project_by_name(self):
-        tenant_ref = PROVIDERS.resource_api.get_project_by_name(
-            self.tenant_bar['name'],
+        project_ref = PROVIDERS.resource_api.get_project_by_name(
+            self.project_bar['name'],
             CONF.identity.default_domain_id)
-        self.assertDictEqual(self.tenant_bar, tenant_ref)
+        self.assertDictEqual(self.project_bar, project_ref)
 
     @unit.skip_if_no_multiple_domains_support
     def test_get_project_by_name_for_project_acting_as_a_domain(self):
@@ -205,42 +206,6 @@ class ResourceTests(object):
                           project['id'],
                           project)
 
-    def test_update_project_blank_name_fails(self):
-        project = unit.new_project_ref(
-            name='fake1', domain_id=CONF.identity.default_domain_id)
-        PROVIDERS.resource_api.create_project(project['id'], project)
-        project['name'] = ''
-        self.assertRaises(exception.ValidationError,
-                          PROVIDERS.resource_api.update_project,
-                          project['id'],
-                          project)
-
-    def test_update_project_long_name_fails(self):
-        project = unit.new_project_ref(
-            name='fake1', domain_id=CONF.identity.default_domain_id)
-        PROVIDERS.resource_api.create_project(project['id'], project)
-        project['name'] = 'a' * 65
-        self.assertRaises(exception.ValidationError,
-                          PROVIDERS.resource_api.update_project,
-                          project['id'],
-                          project)
-
-    def test_update_project_invalid_name_fails(self):
-        project = unit.new_project_ref(
-            name='fake1', domain_id=CONF.identity.default_domain_id)
-        PROVIDERS.resource_api.create_project(project['id'], project)
-        project['name'] = None
-        self.assertRaises(exception.ValidationError,
-                          PROVIDERS.resource_api.update_project,
-                          project['id'],
-                          project)
-
-        project['name'] = 123
-        self.assertRaises(exception.ValidationError,
-                          PROVIDERS.resource_api.update_project,
-                          project['id'],
-                          project)
-
     def test_create_project_invalid_domain_id(self):
         project = unit.new_project_ref(domain_id=uuid.uuid4().hex)
         self.assertRaises(exception.DomainNotFound,
@@ -264,9 +229,9 @@ class ResourceTests(object):
 
     def test_list_projects(self):
         project_refs = PROVIDERS.resource_api.list_projects()
-        project_count = len(default_fixtures.TENANTS) + self.domain_count
+        project_count = len(default_fixtures.PROJECTS) + self.domain_count
         self.assertEqual(project_count, len(project_refs))
-        for project in default_fixtures.TENANTS:
+        for project in default_fixtures.PROJECTS:
             self.assertIn(project, project_refs)
 
     def test_list_projects_with_multiple_filters(self):
@@ -304,11 +269,11 @@ class ResourceTests(object):
         # filtering by domain does not include any project that acts as a
         # domain.
         self.assertThat(
-            project_ids, matchers.HasLength(len(default_fixtures.TENANTS)))
-        self.assertIn(self.tenant_bar['id'], project_ids)
-        self.assertIn(self.tenant_baz['id'], project_ids)
-        self.assertIn(self.tenant_mtu['id'], project_ids)
-        self.assertIn(self.tenant_service['id'], project_ids)
+            project_ids, matchers.HasLength(len(default_fixtures.PROJECTS)))
+        self.assertIn(self.project_bar['id'], project_ids)
+        self.assertIn(self.project_baz['id'], project_ids)
+        self.assertIn(self.project_mtu['id'], project_ids)
+        self.assertIn(self.project_service['id'], project_ids)
 
     @unit.skip_if_no_multiple_domains_support
     def test_list_projects_acting_as_domain(self):
@@ -1294,6 +1259,23 @@ class ResourceTests(object):
                           PROVIDERS.resource_api.get_domain,
                           domain['id'])
 
+    @unit.skip_if_no_multiple_domains_support
+    def test_delete_domain_call_db_time(self):
+        domain = unit.new_domain_ref()
+        PROVIDERS.resource_api.create_domain(domain['id'], domain)
+        # Disable the domain
+        domain['enabled'] = False
+        PROVIDERS.resource_api.update_domain(domain['id'], domain)
+
+        domain_ref = PROVIDERS.resource_api.get_project(domain['id'])
+        with mock.patch.object(resource_sql.Resource,
+                               "get_project") as mock_get_project:
+
+            mock_get_project.return_value = domain_ref
+            # Delete the domain
+            PROVIDERS.resource_api.delete_domain(domain['id'])
+            self.assertEqual(mock_get_project.call_count, 1)
+
     @unit.skip_if_no_multiple_domains_support
     def test_domain_name_case_sensitivity(self):
         # create a ref with a lowercase name
@@ -1713,7 +1695,7 @@ class ResourceDriverTests(object):
         project = {
             'name': uuid.uuid4().hex,
             'id': project_id,
-            'domain_id': uuid.uuid4().hex,
+            'domain_id': default_fixtures.ROOT_DOMAIN['id'],
         }
         self.driver.create_project(project_id, project)
 
@@ -1722,10 +1704,18 @@ class ResourceDriverTests(object):
         project = {
             'name': uuid.uuid4().hex,
             'id': project_id,
-            'domain_id': uuid.uuid4().hex,
+            'domain_id': default_fixtures.ROOT_DOMAIN['id'],
+        }
+        parent_project = self.driver.create_project(project_id, project)
+
+        project_id = uuid.uuid4().hex
+        project = {
+            'name': uuid.uuid4().hex,
+            'id': project_id,
+            'domain_id': default_fixtures.ROOT_DOMAIN['id'],
             'description': uuid.uuid4().hex,
             'enabled': True,
-            'parent_id': uuid.uuid4().hex,
+            'parent_id': parent_project['id'],
             'is_domain': True,
         }
         self.driver.create_project(project_id, project)
@@ -1741,7 +1731,7 @@ class ResourceDriverTests(object):
 
     def test_create_project_same_name_same_domain_conflict(self):
         name = uuid.uuid4().hex
-        domain_id = uuid.uuid4().hex
+        domain_id = default_fixtures.ROOT_DOMAIN['id']
 
         project_id = uuid.uuid4().hex
         project = {
@@ -1766,14 +1756,14 @@ class ResourceDriverTests(object):
         project = {
             'name': uuid.uuid4().hex,
             'id': project_id,
-            'domain_id': uuid.uuid4().hex,
+            'domain_id': default_fixtures.ROOT_DOMAIN['id'],
         }
         self.driver.create_project(project_id, project)
 
         project = {
             'name': uuid.uuid4().hex,
             'id': project_id,
-            'domain_id': uuid.uuid4().hex,
+            'domain_id': default_fixtures.ROOT_DOMAIN['id'],
         }
         self.assertRaises(exception.Conflict, self.driver.create_project,
                           project_id, project)
