@@ -14,6 +14,7 @@ import uuid
 
 from keystone.common import provider_api
 from keystone.tests import unit
+from keystone.tests.unit import default_fixtures
 from keystone.tests.unit.identity.shadow_users import test_backend
 from keystone.tests.unit.identity.shadow_users import test_core
 from keystone.tests.unit.ksfixtures import database
@@ -28,6 +29,8 @@ class ShadowUsersTests(unit.TestCase,
         super(ShadowUsersTests, self).setUp()
         self.useFixture(database.Database())
         self.load_backends()
+        PROVIDERS.resource_api.create_domain(
+            default_fixtures.ROOT_DOMAIN['id'], default_fixtures.ROOT_DOMAIN)
         self.idp = {
             'id': uuid.uuid4().hex,
             'enabled': True,
@@ -47,6 +50,7 @@ class ShadowUsersTests(unit.TestCase,
             'unique_id': uuid.uuid4().hex,
             'display_name': uuid.uuid4().hex
         }
+        self.email = uuid.uuid4().hex
         PROVIDERS.federation_api.create_idp(self.idp['id'], self.idp)
         PROVIDERS.federation_api.create_mapping(
             self.mapping['id'], self.mapping
