@@ -30,7 +30,6 @@ import keystone.conf
 from keystone import exception
 from keystone import oauth1
 from keystone.oauth1.backends import base
-from keystone.oauth1 import controllers
 from keystone.tests import unit
 from keystone.tests.unit.common import test_notifications
 from keystone.tests.unit import ksfixtures
@@ -53,10 +52,8 @@ class OAuth1Tests(test_v3.RestfulTestCase):
 
     def setUp(self):
         super(OAuth1Tests, self).setUp()
-
         # Now that the app has been served, we can query CONF values
         self.base_url = 'http://localhost/v3'
-        self.controller = controllers.OAuthControllerV3()
 
     def _create_single_consumer(self):
         ref = {'description': uuid.uuid4().hex}
@@ -311,6 +308,19 @@ class OAuthFlowTests(OAuth1Tests):
         self.keystone_token = content.result['token']
         self.assertIsNotNone(self.keystone_token_id)
 
+        # add a new role assignment to ensure it is ignored in the access token
+        new_role = {'id': uuid.uuid4().hex, 'name': uuid.uuid4().hex}
+        PROVIDERS.role_api.create_role(new_role['id'], new_role)
+        PROVIDERS.assignment_api.add_role_to_user_and_project(
+            user_id=self.user_id,
+            project_id=self.project_id,
+            role_id=new_role['id'])
+        content = self.post(url, headers=headers, body=body)
+        token = content.result['token']
+        token_roles = [r['id'] for r in token['roles']]
+        self.assertIn(self.role_id, token_roles)
+        self.assertNotIn(new_role['id'], token_roles)
+
 
 class AccessTokenCRUDTests(OAuthFlowTests):
     def test_delete_access_token_dne(self):
@@ -497,11 +507,9 @@ class AuthTokenTests(object):
         r = self.patch('/users/%(user_id)s' % {
             'user_id': self.user['id']},
             body={'user': user})
-
-        headers = {'X-Subject-Token': self.keystone_token_id,
-                   'X-Auth-Token': self.keystone_token_id}
-        self.admin_request(path='/auth/tokens', headers=headers,
-                           method='GET', expected_status=http_client.NOT_FOUND)
+        headers = {'X-Subject-Token': self.keystone_token_id}
+        self.get(path='/auth/tokens', token=self.get_admin_token(),
+                 headers=headers, expected_status=http_client.NOT_FOUND)
 
     def test_deleting_project_also_invalidates_tokens(self):
         self.test_oauth_flow()
@@ -515,10 +523,9 @@ class AuthTokenTests(object):
         r = self.delete('/projects/%(project_id)s' % {
             'project_id': self.project_id})
 
-        headers = {'X-Subject-Token': self.keystone_token_id,
-                   'X-Auth-Token': self.keystone_token_id}
-        self.admin_request(path='/auth/tokens', headers=headers,
-                           method='GET', expected_status=http_client.NOT_FOUND)
+        headers = {'X-Subject-Token': self.keystone_token_id}
+        self.get(path='/auth/tokens', token=self.get_admin_token(),
+                 headers=headers, expected_status=http_client.NOT_FOUND)
 
     def test_token_chaining_is_not_allowed(self):
         self.test_oauth_flow()
@@ -662,13 +669,6 @@ class FernetAuthTokenTests(AuthTokenTests, OAuthFlowTests):
         self.skipTest('Fernet tokens are never persisted in the backend.')
 
 
-class UUIDAuthTokenTests(AuthTokenTests, OAuthFlowTests):
-
-    def config_overrides(self):
-        super(UUIDAuthTokenTests, self).config_overrides()
-        self.config_fixture.config(group='token', provider='uuid')
-
-
 class MaliciousOAuth1Tests(OAuth1Tests):
 
     def _switch_baseurl_scheme(self):
@@ -877,6 +877,16 @@ class MaliciousOAuth1Tests(OAuth1Tests):
         consumer_secret = consumer['secret']
         consumer = {'key': consumer_id, 'secret': consumer_secret}
 
+        # This new role is utilzied to ensure the user still has access to
+        # the project but is authorizing an incorrect role_id for the purposes
+        # of oauth1.
+        new_role = {'id': uuid.uuid4().hex, 'name': uuid.uuid4().hex}
+        PROVIDERS.role_api.create_role(new_role['id'], new_role)
+        PROVIDERS.assignment_api.add_role_to_user_and_project(
+            user_id=self.user_id,
+            project_id=self.project_id,
+            role_id=new_role['id'])
+
         url, headers = self._create_request_token(consumer, self.project_id)
         content = self.post(
             url, headers=headers,
@@ -885,11 +895,14 @@ class MaliciousOAuth1Tests(OAuth1Tests):
         request_key = credentials['oauth_token'][0]
 
         PROVIDERS.assignment_api.remove_role_from_user_and_project(
-            self.user_id, self.project_id, self.role_id)
+            self.user_id, self.project_id, new_role['id'])
         url = self._authorize_request_token(request_key)
-        body = {'roles': [{'id': self.role_id}]}
-        self.admin_request(path=url, method='PUT',
-                           body=body, expected_status=http_client.NOT_FOUND)
+        body = {'roles': [{'id': new_role['id']}]}
+        # NOTE(morgan): previous versions of this test erroneously checked for
+        # 404 because an unrouted URI was being hit. It is correct to get a 401
+        # error back as the role is not in the superset of roles the user
+        # has at the time of the Authorization.
+        self.put(path=url, body=body, expected_status=http_client.UNAUTHORIZED)
 
     def test_bad_authorizing_roles_name(self):
         consumer = self._create_single_consumer()
@@ -906,8 +919,7 @@ class MaliciousOAuth1Tests(OAuth1Tests):
 
         url = self._authorize_request_token(request_key)
         body = {'roles': [{'name': 'fake_name'}]}
-        self.admin_request(path=url, method='PUT',
-                           body=body, expected_status=http_client.NOT_FOUND)
+        self.put(path=url, body=body, expected_status=http_client.NOT_FOUND)
 
     def test_no_authorizing_user_id(self):
         consumer = self._create_single_consumer()
