@@ -21,16 +21,17 @@ from oslo_utils import timeutils
 import six
 
 from keystone import auth
+from keystone.common import fernet_utils
 from keystone.common import provider_api
-from keystone.common import token_utils
 from keystone.common import utils
 import keystone.conf
 from keystone import exception
 from keystone.federation import constants as federation_constants
 from keystone.tests import unit
+from keystone.tests.unit import default_fixtures
 from keystone.tests.unit import ksfixtures
 from keystone.tests.unit.ksfixtures import database
-from keystone.token.providers import common
+from keystone.token import provider
 from keystone.token.providers import fernet
 from keystone.token import token_formatters
 
@@ -42,50 +43,23 @@ PROVIDERS = provider_api.ProviderAPIs
 class TestFernetTokenProvider(unit.TestCase):
     def setUp(self):
         super(TestFernetTokenProvider, self).setUp()
-        self.useFixture(
-            ksfixtures.KeyRepository(
-                self.config_fixture,
-                'fernet_tokens',
-                CONF.fernet_tokens.max_active_keys
-            )
-        )
         self.provider = fernet.Provider()
 
-    def test_supports_bind_authentication_returns_false(self):
-        self.assertFalse(self.provider._supports_bind_authentication)
-
-    def test_needs_persistence_returns_false(self):
-        self.assertFalse(self.provider.needs_persistence())
-
-    def test_invalid_v3_token_raises_token_not_found(self):
+    def test_invalid_token_raises_token_not_found(self):
         token_id = uuid.uuid4().hex
-        e = self.assertRaises(
+        self.assertRaises(
             exception.TokenNotFound,
             self.provider.validate_token,
             token_id)
-        self.assertIn(token_id, u'%s' % e)
-
-    def test_invalid_v2_token_raises_token_not_found(self):
-        token_id = uuid.uuid4().hex
-        e = self.assertRaises(
-            exception.TokenNotFound,
-            self.provider.validate_token,
-            token_id)
-        self.assertIn(token_id, u'%s' % e)
 
 
 class TestValidate(unit.TestCase):
     def setUp(self):
         super(TestValidate, self).setUp()
-        self.useFixture(
-            ksfixtures.KeyRepository(
-                self.config_fixture,
-                'fernet_tokens',
-                CONF.fernet_tokens.max_active_keys
-            )
-        )
         self.useFixture(database.Database())
         self.load_backends()
+        PROVIDERS.resource_api.create_domain(
+            default_fixtures.ROOT_DOMAIN['id'], default_fixtures.ROOT_DOMAIN)
 
     def config_overrides(self):
         super(TestValidate, self).config_overrides()
@@ -104,25 +78,20 @@ class TestValidate(unit.TestCase):
         user_ref = PROVIDERS.identity_api.create_user(user_ref)
 
         method_names = ['password']
-        token_id, token_data_ = PROVIDERS.token_provider_api.issue_token(
+        token = PROVIDERS.token_provider_api.issue_token(
             user_ref['id'], method_names)
 
-        token_data = PROVIDERS.token_provider_api.validate_token(token_id)
-        token = token_data['token']
-        self.assertIsInstance(token['audit_ids'], list)
-        self.assertIsInstance(token['expires_at'], str)
-        self.assertIsInstance(token['issued_at'], str)
-        self.assertEqual(method_names, token['methods'])
-        exp_user_info = {
-            'id': user_ref['id'],
-            'name': user_ref['name'],
-            'domain': {
-                'id': domain_ref['id'],
-                'name': domain_ref['name'],
-            },
-            'password_expires_at': user_ref['password_expires_at']
-        }
-        self.assertEqual(exp_user_info, token['user'])
+        token = PROVIDERS.token_provider_api.validate_token(token.id)
+        self.assertIsInstance(token.audit_ids, list)
+        self.assertIsInstance(token.expires_at, str)
+        self.assertIsInstance(token.issued_at, str)
+        self.assertEqual(method_names, token.methods)
+        self.assertEqual(user_ref['id'], token.user_id)
+        self.assertEqual(user_ref['name'], token.user['name'])
+        self.assertDictEqual(domain_ref, token.user_domain)
+        self.assertEqual(
+            user_ref['password_expires_at'], token.user['password_expires_at']
+        )
 
     def test_validate_v3_token_federated_info(self):
         # Check the user fields in the token result when use validate_v3_token
@@ -155,23 +124,18 @@ class TestValidate(unit.TestCase):
             federation_constants.PROTOCOL: protocol,
         }
         auth_context = auth.core.AuthContext(**auth_context_params)
-        token_id, token_data_ = PROVIDERS.token_provider_api.issue_token(
+        token = PROVIDERS.token_provider_api.issue_token(
             user_ref['id'], method_names, auth_context=auth_context)
 
-        token_data = PROVIDERS.token_provider_api.validate_token(token_id)
-        token = token_data['token']
-        exp_user_info = {
-            'id': user_ref['id'],
-            'name': user_ref['name'],
-            'domain': {'id': CONF.federation.federated_domain_name,
-                       'name': CONF.federation.federated_domain_name, },
-            federation_constants.FEDERATION: {
-                'groups': [{'id': group_id} for group_id in group_ids],
-                'identity_provider': {'id': idp_id, },
-                'protocol': {'id': protocol, },
-            },
-        }
-        self.assertDictEqual(exp_user_info, token['user'])
+        token = PROVIDERS.token_provider_api.validate_token(token.id)
+
+        self.assertEqual(user_ref['id'], token.user_id)
+        self.assertEqual(user_ref['name'], token.user['name'])
+        self.assertDictEqual(domain_ref, token.user_domain)
+        exp_group_ids = [{'id': group_id} for group_id in group_ids]
+        self.assertEqual(exp_group_ids, token.federated_groups)
+        self.assertEqual(idp_id, token.identity_provider_id)
+        self.assertEqual(protocol, token.protocol_id)
 
     def test_validate_v3_token_trust(self):
         # Check the trust fields in the token result when use validate_v3_token
@@ -215,19 +179,15 @@ class TestValidate(unit.TestCase):
 
         method_names = ['password']
 
-        token_id, token_data_ = PROVIDERS.token_provider_api.issue_token(
+        token = PROVIDERS.token_provider_api.issue_token(
             user_ref['id'], method_names, project_id=project_ref['id'],
-            trust=trust_ref)
+            trust_id=trust_ref['id'])
 
-        token_data = PROVIDERS.token_provider_api.validate_token(token_id)
-        token = token_data['token']
-        exp_trust_info = {
-            'id': trust_ref['id'],
-            'impersonation': False,
-            'trustee_user': {'id': user_ref['id'], },
-            'trustor_user': {'id': trustor_user_ref['id'], },
-        }
-        self.assertEqual(exp_trust_info, token['OS-TRUST:trust'])
+        token = PROVIDERS.token_provider_api.validate_token(token.id)
+        self.assertEqual(trust_ref['id'], token.trust_id)
+        self.assertFalse(token.trust['impersonation'])
+        self.assertEqual(user_ref['id'], token.trustee['id'])
+        self.assertEqual(trustor_user_ref['id'], token.trustor['id'])
 
     def test_validate_v3_token_validation_error_exc(self):
         # When the token format isn't recognized, TokenNotFound is raised.
@@ -242,16 +202,6 @@ class TestValidate(unit.TestCase):
 
 
 class TestTokenFormatter(unit.TestCase):
-    def setUp(self):
-        super(TestTokenFormatter, self).setUp()
-        self.useFixture(
-            ksfixtures.KeyRepository(
-                self.config_fixture,
-                'fernet_tokens',
-                CONF.fernet_tokens.max_active_keys
-            )
-        )
-
     def test_restore_padding(self):
         # 'a' will result in '==' padding, 'aa' will result in '=' padding, and
         # 'aaa' will result in no padding.
@@ -288,7 +238,7 @@ class TestPayloads(unit.TestCase):
                                                        delta=1e-05)
 
     def test_strings_can_be_converted_to_bytes(self):
-        s = common.random_urlsafe_str()
+        s = provider.random_urlsafe_str()
         self.assertIsInstance(s, six.text_type)
 
         b = token_formatters.BasePayload.random_urlsafe_str_to_bytes(s)
@@ -337,24 +287,77 @@ class TestPayloads(unit.TestCase):
             actual_time_float)
         self.assertEqual(expected_time_str, actual_time_str)
 
+    def test_convert_or_decode_uuid_bytes(self):
+        payload_cls = token_formatters.BasePayload
+
+        expected_hex_uuid = uuid.uuid4().hex
+        uuid_obj = uuid.UUID(expected_hex_uuid)
+        expected_uuid_in_bytes = uuid_obj.bytes
+
+        actual_hex_uuid = payload_cls._convert_or_decode(
+            is_stored_as_bytes=True,
+            value=expected_uuid_in_bytes
+        )
+
+        self.assertEqual(expected_hex_uuid, actual_hex_uuid)
+
+    def test_convert_or_decode_binary_type(self):
+        payload_cls = token_formatters.BasePayload
+
+        expected_hex_uuid = uuid.uuid4().hex
+
+        actual_hex_uuid = payload_cls._convert_or_decode(
+            is_stored_as_bytes=False,
+            value=expected_hex_uuid.encode('utf-8')
+        )
+
+        self.assertEqual(expected_hex_uuid, actual_hex_uuid)
+
+    def test_convert_or_decode_text_type(self):
+        payload_cls = token_formatters.BasePayload
+
+        expected_hex_uuid = uuid.uuid4().hex
+
+        actual_hex_uuid = payload_cls._convert_or_decode(
+            is_stored_as_bytes=False,
+            value=expected_hex_uuid
+        )
+
+        self.assertEqual(expected_hex_uuid, actual_hex_uuid)
+
     def _test_payload(self, payload_class, exp_user_id=None, exp_methods=None,
-                      exp_system=None, exp_project_id=None,
-                      exp_domain_id=None, exp_trust_id=None,
-                      exp_federated_info=None, exp_access_token_id=None,
-                      exp_app_cred_id=None):
+                      exp_system=None, exp_project_id=None, exp_domain_id=None,
+                      exp_trust_id=None, exp_federated_group_ids=None,
+                      exp_identity_provider_id=None, exp_protocol_id=None,
+                      exp_access_token_id=None, exp_app_cred_id=None,
+                      encode_ids=False):
+        def _encode_id(value):
+            if value is not None and six.text_type(value) and encode_ids:
+                return value.encode('utf-8')
+            return value
         exp_user_id = exp_user_id or uuid.uuid4().hex
         exp_methods = exp_methods or ['password']
         exp_expires_at = utils.isotime(timeutils.utcnow(), subsecond=True)
-        exp_audit_ids = [common.random_urlsafe_str()]
+        exp_audit_ids = [provider.random_urlsafe_str()]
 
         payload = payload_class.assemble(
-            exp_user_id, exp_methods, exp_system, exp_project_id,
-            exp_domain_id, exp_expires_at, exp_audit_ids, exp_trust_id,
-            exp_federated_info, exp_access_token_id, exp_app_cred_id)
+            _encode_id(exp_user_id),
+            exp_methods,
+            _encode_id(exp_system),
+            _encode_id(exp_project_id),
+            exp_domain_id,
+            exp_expires_at,
+            exp_audit_ids,
+            exp_trust_id,
+            _encode_id(exp_federated_group_ids),
+            _encode_id(exp_identity_provider_id),
+            exp_protocol_id,
+            _encode_id(exp_access_token_id),
+            _encode_id(exp_app_cred_id))
 
         (user_id, methods, system, project_id,
          domain_id, expires_at, audit_ids,
-         trust_id, federated_info,
+         trust_id, federated_group_ids, identity_provider_id, protocol_id,
          access_token_id, app_cred_id) = payload_class.disassemble(payload)
 
         self.assertEqual(exp_user_id, user_id)
@@ -364,15 +367,13 @@ class TestPayloads(unit.TestCase):
         self.assertEqual(exp_system, system)
         self.assertEqual(exp_project_id, project_id)
         self.assertEqual(exp_domain_id, domain_id)
+        self.assertEqual(exp_federated_group_ids, federated_group_ids)
+        self.assertEqual(exp_identity_provider_id, identity_provider_id)
+        self.assertEqual(exp_protocol_id, protocol_id)
         self.assertEqual(exp_trust_id, trust_id)
         self.assertEqual(exp_access_token_id, access_token_id)
         self.assertEqual(exp_app_cred_id, app_cred_id)
 
-        if exp_federated_info:
-            self.assertDictEqual(exp_federated_info, federated_info)
-        else:
-            self.assertIsNone(federated_info)
-
     def test_unscoped_payload(self):
         self._test_payload(token_formatters.UnscopedPayload)
 
@@ -415,6 +416,12 @@ class TestPayloads(unit.TestCase):
                            exp_user_id='0123456789abcdef',
                            exp_project_id='0123456789abcdef')
 
+    def test_project_scoped_payload_with_binary_encoded_ids(self):
+        self._test_payload(token_formatters.ProjectScopedPayload,
+                           exp_user_id='someNonUuidUserId',
+                           exp_project_id='someNonUuidProjectId',
+                           encode_ids=True)
+
     def test_domain_scoped_payload_with_non_uuid_user_id(self):
         self._test_payload(token_formatters.DomainScopedPayload,
                            exp_user_id='nonUuidUserId',
@@ -438,13 +445,15 @@ class TestPayloads(unit.TestCase):
                            exp_trust_id=uuid.uuid4().hex)
 
     def _test_federated_payload_with_ids(self, exp_user_id, exp_group_id):
-        exp_federated_info = {'group_ids': [{'id': exp_group_id}],
-                              'idp_id': uuid.uuid4().hex,
-                              'protocol_id': uuid.uuid4().hex}
+        exp_federated_group_ids = [{'id': exp_group_id}]
+        exp_idp_id = uuid.uuid4().hex
+        exp_protocol_id = uuid.uuid4().hex
 
         self._test_payload(token_formatters.FederatedUnscopedPayload,
                            exp_user_id=exp_user_id,
-                           exp_federated_info=exp_federated_info)
+                           exp_federated_group_ids=exp_federated_group_ids,
+                           exp_identity_provider_id=exp_idp_id,
+                           exp_protocol_id=exp_protocol_id)
 
     def test_federated_payload_with_non_uuid_ids(self):
         self._test_federated_payload_with_ids('someNonUuidUserId',
@@ -455,26 +464,30 @@ class TestPayloads(unit.TestCase):
                                               '0123456789abcdef')
 
     def test_federated_project_scoped_payload(self):
-        exp_federated_info = {'group_ids': [{'id': 'someNonUuidGroupId'}],
-                              'idp_id': uuid.uuid4().hex,
-                              'protocol_id': uuid.uuid4().hex}
+        exp_federated_group_ids = [{'id': 'someNonUuidGroupId'}]
+        exp_idp_id = uuid.uuid4().hex
+        exp_protocol_id = uuid.uuid4().hex
 
         self._test_payload(token_formatters.FederatedProjectScopedPayload,
                            exp_user_id='someNonUuidUserId',
                            exp_methods=['token'],
                            exp_project_id=uuid.uuid4().hex,
-                           exp_federated_info=exp_federated_info)
+                           exp_federated_group_ids=exp_federated_group_ids,
+                           exp_identity_provider_id=exp_idp_id,
+                           exp_protocol_id=exp_protocol_id)
 
     def test_federated_domain_scoped_payload(self):
-        exp_federated_info = {'group_ids': [{'id': 'someNonUuidGroupId'}],
-                              'idp_id': uuid.uuid4().hex,
-                              'protocol_id': uuid.uuid4().hex}
+        exp_federated_group_ids = [{'id': 'someNonUuidGroupId'}]
+        exp_idp_id = uuid.uuid4().hex
+        exp_protocol_id = uuid.uuid4().hex
 
         self._test_payload(token_formatters.FederatedDomainScopedPayload,
                            exp_user_id='someNonUuidUserId',
                            exp_methods=['token'],
                            exp_domain_id=uuid.uuid4().hex,
-                           exp_federated_info=exp_federated_info)
+                           exp_federated_group_ids=exp_federated_group_ids,
+                           exp_identity_provider_id=exp_idp_id,
+                           exp_protocol_id=exp_protocol_id)
 
     def test_oauth_scoped_payload(self):
         self._test_payload(token_formatters.OauthScopedPayload,
@@ -526,7 +539,7 @@ class TestFernetKeyRotation(unit.TestCase):
 
         """
         # Load the keys into a list, keys is list of six.text_type.
-        key_utils = token_utils.TokenUtils(
+        key_utils = fernet_utils.FernetUtils(
             CONF.fernet_tokens.key_repository,
             CONF.fernet_tokens.max_active_keys,
             'fernet_tokens'
@@ -594,7 +607,7 @@ class TestFernetKeyRotation(unit.TestCase):
 
             # Rotate the keys just enough times to fully populate the key
             # repository.
-            key_utils = token_utils.TokenUtils(
+            key_utils = fernet_utils.FernetUtils(
                 CONF.fernet_tokens.key_repository,
                 CONF.fernet_tokens.max_active_keys,
                 'fernet_tokens'
@@ -612,7 +625,7 @@ class TestFernetKeyRotation(unit.TestCase):
 
             # Rotate an additional number of times to ensure that we maintain
             # the desired number of active keys.
-            key_utils = token_utils.TokenUtils(
+            key_utils = fernet_utils.FernetUtils(
                 CONF.fernet_tokens.key_repository,
                 CONF.fernet_tokens.max_active_keys,
                 'fernet_tokens'
@@ -627,19 +640,10 @@ class TestFernetKeyRotation(unit.TestCase):
                 self.assertEqual(exp_keys, self.keys)
 
     def test_rotation_disk_write_fail(self):
-        # Init the key repository
-        self.useFixture(
-            ksfixtures.KeyRepository(
-                self.config_fixture,
-                'fernet_tokens',
-                CONF.fernet_tokens.max_active_keys
-            )
-        )
-
         # Make sure that the init key repository contains 2 keys
         self.assertRepositoryState(expected_size=2)
 
-        key_utils = token_utils.TokenUtils(
+        key_utils = fernet_utils.FernetUtils(
             CONF.fernet_tokens.key_repository,
             CONF.fernet_tokens.max_active_keys,
             'fernet_tokens'
@@ -650,13 +654,13 @@ class TestFernetKeyRotation(unit.TestCase):
         file_handle = mock_open()
         file_handle.flush.side_effect = IOError('disk full')
 
-        with mock.patch('keystone.common.token_utils.open', mock_open):
+        with mock.patch('keystone.common.fernet_utils.open', mock_open):
             self.assertRaises(IOError, key_utils.rotate_keys)
 
         # Assert that the key repository is unchanged
         self.assertEqual(self.key_repository_size, 2)
 
-        with mock.patch('keystone.common.token_utils.open', mock_open):
+        with mock.patch('keystone.common.fernet_utils.open', mock_open):
             self.assertRaises(IOError, key_utils.rotate_keys)
 
         # Assert that the key repository is still unchanged, even after
@@ -670,18 +674,29 @@ class TestFernetKeyRotation(unit.TestCase):
         # Assert that the key repository is now expanded.
         self.assertEqual(self.key_repository_size, 3)
 
-    def test_non_numeric_files(self):
-        self.useFixture(
-            ksfixtures.KeyRepository(
-                self.config_fixture,
-                'fernet_tokens',
-                CONF.fernet_tokens.max_active_keys
-            )
+    def test_rotation_empty_file(self):
+        active_keys = 2
+        self.assertRepositoryState(expected_size=active_keys)
+        empty_file = os.path.join(CONF.fernet_tokens.key_repository, '2')
+        with open(empty_file, 'w'):
+            pass
+        key_utils = fernet_utils.FernetUtils(
+            CONF.fernet_tokens.key_repository,
+            CONF.fernet_tokens.max_active_keys,
+            'fernet_tokens'
         )
+        # Rotate the keys to overwrite the empty file
+        key_utils.rotate_keys()
+        self.assertTrue(os.path.isfile(empty_file))
+        keys = key_utils.load_keys()
+        self.assertEqual(3, len(keys))
+        self.assertTrue(os.path.getsize(empty_file) > 0)
+
+    def test_non_numeric_files(self):
         evil_file = os.path.join(CONF.fernet_tokens.key_repository, '99.bak')
         with open(evil_file, 'w'):
             pass
-        key_utils = token_utils.TokenUtils(
+        key_utils = fernet_utils.FernetUtils(
             CONF.fernet_tokens.key_repository,
             CONF.fernet_tokens.max_active_keys,
             'fernet_tokens'
@@ -698,26 +713,34 @@ class TestFernetKeyRotation(unit.TestCase):
 
 class TestLoadKeys(unit.TestCase):
 
-    def assertValidFernetKey(self, keys):
-        self.assertGreater(len(keys[0]), 0)
-        self.assertIsInstance(keys[0], str)
+    def assertValidFernetKeys(self, keys):
+        # Make sure each key is a non-empty string
+        for key in keys:
+            self.assertGreater(len(key), 0)
+            self.assertIsInstance(key, str)
 
     def test_non_numeric_files(self):
-        self.useFixture(
-            ksfixtures.KeyRepository(
-                self.config_fixture,
-                'fernet_tokens',
-                CONF.fernet_tokens.max_active_keys
-            )
-        )
         evil_file = os.path.join(CONF.fernet_tokens.key_repository, '~1')
         with open(evil_file, 'w'):
             pass
-        key_utils = token_utils.TokenUtils(
+        key_utils = fernet_utils.FernetUtils(
             CONF.fernet_tokens.key_repository,
             CONF.fernet_tokens.max_active_keys,
             'fernet_tokens'
         )
         keys = key_utils.load_keys()
         self.assertEqual(2, len(keys))
-        self.assertValidFernetKey(keys)
+        self.assertValidFernetKeys(keys)
+
+    def test_empty_files(self):
+        empty_file = os.path.join(CONF.fernet_tokens.key_repository, '2')
+        with open(empty_file, 'w'):
+            pass
+        key_utils = fernet_utils.FernetUtils(
+            CONF.fernet_tokens.key_repository,
+            CONF.fernet_tokens.max_active_keys,
+            'fernet_tokens'
+        )
+        keys = key_utils.load_keys()
+        self.assertEqual(2, len(keys))
+        self.assertValidFernetKeys(keys)
